@@ -12,7 +12,16 @@ import cbor2
 import pytest
 from cbor2 import CBORTag
 
-from cwt import CWT, COSEKey, DecodeError, EncodeError, Recipient, VerifyError, cose_key
+from cwt import (
+    CWT,
+    COSEKey,
+    DecodeError,
+    EncodeError,
+    Recipient,
+    VerifyError,
+    claims,
+    cose_key,
+)
 
 from .utils import key_path, now
 
@@ -58,6 +67,19 @@ class TestCWT:
             pytest.fail("CWT() should fail.")
         assert "should be" in str(err.value)
 
+    def test_cwt_encode_with_claims_object(self, ctx):
+        key = cose_key.from_symmetric_key(alg="HS256")
+        token = ctx.encode(
+            claims.from_json(
+                {"iss": "https://as.example", "sub": "someone", "cti": b"123"}
+            ),
+            key,
+        )
+        decoded = ctx.decode(token, key)
+        assert 1 in decoded and decoded[1] == "https://as.example"
+        assert 2 in decoded and decoded[2] == "someone"
+        assert 7 in decoded and decoded[7] == b"123"
+
     @pytest.mark.parametrize(
         "invalid_key",
         [
@@ -367,6 +389,37 @@ class TestCWT:
         assert 2 in decoded and decoded[2] == "someone"
         assert 7 in decoded and decoded[7] == b"123"
 
+    @pytest.mark.parametrize(
+        "alg",
+        [
+            "PS256",
+            "PS384",
+            "PS512",
+            "RS256",
+            "RS384",
+            "RS512",
+            -37,
+            -38,
+            -39,
+            -257,
+            -258,
+            -259,
+        ],
+    )
+    def test_cwt_encode_and_sign_with_valid_alg_rsa(self, ctx, alg):
+        with open(key_path("public_key_rsa.pem")) as key_file:
+            public_key = cose_key.from_pem(key_file.read(), alg=alg)
+        with open(key_path("private_key_rsa.pem")) as key_file:
+            private_key = cose_key.from_pem(key_file.read(), alg=alg)
+        token = ctx.encode_and_sign(
+            {1: "https://as.example", 2: "someone", 7: b"123"},
+            private_key,
+        )
+        decoded = ctx.decode(token, public_key)
+        assert 1 in decoded and decoded[1] == "https://as.example"
+        assert 2 in decoded and decoded[2] == "someone"
+        assert 7 in decoded and decoded[7] == b"123"
+
     def test_cwt_encode_and_sign_with_tagged(self, ctx):
         with open(key_path("private_key_es256.pem")) as key_file:
             private_key = cose_key.from_pem(key_file.read())
