@@ -6,10 +6,11 @@
 """
 Tests for samples on README and RFCs related to CWT/COSE.
 """
-# import cbor2
-# import pytest
 from secrets import token_bytes
 
+# import cbor2
+import pytest
+
 import cwt
 from cwt import claims, cose_key
 
@@ -82,12 +83,12 @@ class TestSample:
         key = cose_key.from_symmetric_key("mysecretpassword")
         encoded = cwt.encode_and_mac(
             claims.from_json(
-                {"iss": "https://as.example", "sub": "dajiaji", "cti": "123"}
+                {"iss": "coaps://as.example", "sub": "dajiaji", "cti": "123"}
             ),
             key,
         )
         decoded = cwt.decode(encoded, key)
-        assert 1 in decoded and decoded[1] == "https://as.example"
+        assert 1 in decoded and decoded[1] == "coaps://as.example"
         assert 2 in decoded and decoded[2] == "dajiaji"
         assert 4 in decoded and decoded[4] <= now() + 3600
         assert 5 in decoded and decoded[5] <= now()
@@ -97,62 +98,62 @@ class TestSample:
     def test_sample_readme_maced_cwt_with_json_dict(self):
         key = cose_key.from_symmetric_key(alg="HMAC 256/256")
         token = cwt.encode(
-            {"iss": "https://as.example", "sub": "dajiaji", "cti": "123"}, key
+            {"iss": "coaps://as.example", "sub": "dajiaji", "cti": "123"}, key
         )
         decoded = cwt.decode(token, key)
-        assert 1 in decoded and decoded[1] == "https://as.example"
+        assert 1 in decoded and decoded[1] == "coaps://as.example"
 
     def test_sample_readme_maced_cwt_with_json_str_old(self):
         key = cose_key.from_symmetric_key("mysecretpassword")
         encoded = cwt.encode_and_mac(
             claims.from_json(
-                '{"iss":"https://as.example","sub":"dajiaji","cti":"123"}'
+                '{"iss":"coaps://as.example","sub":"dajiaji","cti":"123"}'
             ),
             key,
         )
         decoded = cwt.decode(encoded, key)
-        assert 1 in decoded and decoded[1] == "https://as.example"
+        assert 1 in decoded and decoded[1] == "coaps://as.example"
 
     def test_sample_readme_maced_cwt_with_json_str(self):
         key = cose_key.from_symmetric_key(alg="HMAC 256/256")
         token = cwt.encode(
-            '{"iss":"https://as.example","sub":"dajiaji","cti":"123"}', key
+            '{"iss":"coaps://as.example","sub":"dajiaji","cti":"123"}', key
         )
         decoded = cwt.decode(token, key)
-        assert 1 in decoded and decoded[1] == "https://as.example"
+        assert 1 in decoded and decoded[1] == "coaps://as.example"
 
     def test_sample_readme_maced_cwt_with_json_bytes_old(self):
         key = cose_key.from_symmetric_key("mysecretpassword")
         encoded = cwt.encode_and_mac(
             claims.from_json(
-                b'{"iss":"https://as.example","sub":"dajiaji","cti":"123"}'
+                b'{"iss":"coaps://as.example","sub":"dajiaji","cti":"123"}'
             ),
             key,
         )
         decoded = cwt.decode(encoded, key)
-        assert 1 in decoded and decoded[1] == "https://as.example"
+        assert 1 in decoded and decoded[1] == "coaps://as.example"
 
     def test_sample_readme_maced_cwt_with_json_bytes(self):
         key = cose_key.from_symmetric_key(alg="HMAC 256/256")
         token = cwt.encode(
-            b'{"iss":"https://as.example","sub":"dajiaji","cti":"123"}', key
+            b'{"iss":"coaps://as.example","sub":"dajiaji","cti":"123"}', key
         )
         decoded = cwt.decode(token, key)
-        assert 1 in decoded and decoded[1] == "https://as.example"
+        assert 1 in decoded and decoded[1] == "coaps://as.example"
 
     def test_sample_readme_maced_cwt_old(self):
         key = cose_key.from_symmetric_key("mysecretpassword")
         encoded = cwt.encode_and_mac(
-            {1: "https://as.example", 2: "dajiaji", 7: b"123"}, key
+            {1: "coaps://as.example", 2: "dajiaji", 7: b"123"}, key
         )
         decoded = cwt.decode(encoded, key)
-        assert 1 in decoded and decoded[1] == "https://as.example"
+        assert 1 in decoded and decoded[1] == "coaps://as.example"
 
     def test_sample_readme_maced_cwt(self):
         key = cose_key.from_symmetric_key(alg="HMAC 256/256")
-        token = cwt.encode({1: "https://as.example", 2: "dajiaji", 7: b"123"}, key)
+        token = cwt.encode({1: "coaps://as.example", 2: "dajiaji", 7: b"123"}, key)
         decoded = cwt.decode(token, key)
-        assert 1 in decoded and decoded[1] == "https://as.example"
+        assert 1 in decoded and decoded[1] == "coaps://as.example"
 
     def test_sample_readme_signed_cwt_es256_old(self):
         with open(key_path("private_key_es256.pem")) as key_file:
@@ -162,13 +163,13 @@ class TestSample:
 
         encoded = cwt.encode_and_sign(
             claims.from_json(
-                {"iss": "https://as.example", "sub": "dajiaji", "cti": "123"}
+                {"iss": "coaps://as.example", "sub": "dajiaji", "cti": "123"}
             ),
             private_key,
         )
 
         decoded = cwt.decode(encoded, public_key)
-        assert 1 in decoded and decoded[1] == "https://as.example"
+        assert 1 in decoded and decoded[1] == "coaps://as.example"
 
     def test_sample_readme_signed_cwt_es256(self):
         with open(key_path("private_key_es256.pem")) as key_file:
@@ -177,11 +178,11 @@ class TestSample:
             public_key = cose_key.from_pem(key_file.read())
 
         token = cwt.encode(
-            {"iss": "https://as.example", "sub": "dajiaji", "cti": "123"}, private_key
+            {"iss": "coaps://as.example", "sub": "dajiaji", "cti": "123"}, private_key
         )
 
         decoded = cwt.decode(token, public_key)
-        assert 1 in decoded and decoded[1] == "https://as.example"
+        assert 1 in decoded and decoded[1] == "coaps://as.example"
 
     def test_sample_readme_signed_cwt_es384_old(self):
         with open(key_path("private_key_es384.pem")) as key_file:
@@ -191,13 +192,13 @@ class TestSample:
 
         encoded = cwt.encode_and_sign(
             claims.from_json(
-                {"iss": "https://as.example", "sub": "dajiaji", "cti": "123"}
+                {"iss": "coaps://as.example", "sub": "dajiaji", "cti": "123"}
             ),
             private_key,
         )
 
         decoded = cwt.decode(encoded, public_key)
-        assert 1 in decoded and decoded[1] == "https://as.example"
+        assert 1 in decoded and decoded[1] == "coaps://as.example"
 
     def test_sample_readme_signed_cwt_es384(self):
         with open(key_path("private_key_es384.pem")) as key_file:
@@ -206,11 +207,11 @@ class TestSample:
             public_key = cose_key.from_pem(key_file.read())
 
         token = cwt.encode(
-            {"iss": "https://as.example", "sub": "dajiaji", "cti": "123"}, private_key
+            {"iss": "coaps://as.example", "sub": "dajiaji", "cti": "123"}, private_key
         )
 
         decoded = cwt.decode(token, public_key)
-        assert 1 in decoded and decoded[1] == "https://as.example"
+        assert 1 in decoded and decoded[1] == "coaps://as.example"
 
     def test_sample_readme_signed_cwt_es512_old(self):
         with open(key_path("private_key_es512.pem")) as key_file:
@@ -220,13 +221,13 @@ class TestSample:
 
         encoded = cwt.encode_and_sign(
             claims.from_json(
-                {"iss": "https://as.example", "sub": "dajiaji", "cti": "123"}
+                {"iss": "coaps://as.example", "sub": "dajiaji", "cti": "123"}
             ),
             private_key,
         )
 
         decoded = cwt.decode(encoded, public_key)
-        assert 1 in decoded and decoded[1] == "https://as.example"
+        assert 1 in decoded and decoded[1] == "coaps://as.example"
 
     def test_sample_readme_signed_cwt_es512(self):
         with open(key_path("private_key_es512.pem")) as key_file:
@@ -235,11 +236,11 @@ class TestSample:
             public_key = cose_key.from_pem(key_file.read())
 
         token = cwt.encode(
-            {"iss": "https://as.example", "sub": "dajiaji", "cti": "123"}, private_key
+            {"iss": "coaps://as.example", "sub": "dajiaji", "cti": "123"}, private_key
         )
 
         decoded = cwt.decode(token, public_key)
-        assert 1 in decoded and decoded[1] == "https://as.example"
+        assert 1 in decoded and decoded[1] == "coaps://as.example"
 
     def test_sample_readme_signed_cwt_es256k_old(self):
         with open(key_path("private_key_es256k.pem")) as key_file:
@@ -249,13 +250,13 @@ class TestSample:
 
         encoded = cwt.encode_and_sign(
             claims.from_json(
-                {"iss": "https://as.example", "sub": "dajiaji", "cti": "123"}
+                {"iss": "coaps://as.example", "sub": "dajiaji", "cti": "123"}
             ),
             private_key,
         )
 
         decoded = cwt.decode(encoded, public_key)
-        assert 1 in decoded and decoded[1] == "https://as.example"
+        assert 1 in decoded and decoded[1] == "coaps://as.example"
 
     def test_sample_readme_signed_cwt_es256k(self):
         with open(key_path("private_key_es256k.pem")) as key_file:
@@ -264,11 +265,11 @@ class TestSample:
             public_key = cose_key.from_pem(key_file.read())
 
         token = cwt.encode(
-            {"iss": "https://as.example", "sub": "dajiaji", "cti": "123"}, private_key
+            {"iss": "coaps://as.example", "sub": "dajiaji", "cti": "123"}, private_key
         )
 
         decoded = cwt.decode(token, public_key)
-        assert 1 in decoded and decoded[1] == "https://as.example"
+        assert 1 in decoded and decoded[1] == "coaps://as.example"
 
     def test_sample_readme_signed_cwt_ed25519_old(self):
         with open(key_path("private_key_ed25519.pem")) as key_file:
@@ -278,13 +279,13 @@ class TestSample:
 
         encoded = cwt.encode_and_sign(
             claims.from_json(
-                {"iss": "https://as.example", "sub": "dajiaji", "cti": "123"}
+                {"iss": "coaps://as.example", "sub": "dajiaji", "cti": "123"}
             ),
             private_key,
         )
 
         decoded = cwt.decode(encoded, public_key)
-        assert 1 in decoded and decoded[1] == "https://as.example"
+        assert 1 in decoded and decoded[1] == "coaps://as.example"
 
     def test_sample_readme_signed_cwt_ed25519(self):
         with open(key_path("private_key_ed25519.pem")) as key_file:
@@ -293,11 +294,37 @@ class TestSample:
             public_key = cose_key.from_pem(key_file.read())
 
         token = cwt.encode(
-            {"iss": "https://as.example", "sub": "dajiaji", "cti": "123"}, private_key
+            {"iss": "coaps://as.example", "sub": "dajiaji", "cti": "123"}, private_key
         )
 
         decoded = cwt.decode(token, public_key)
-        assert 1 in decoded and decoded[1] == "https://as.example"
+        assert 1 in decoded and decoded[1] == "coaps://as.example"
+
+    def test_sample_readme_signed_cwt_rs256(self):
+        with open(key_path("private_key_rsa.pem")) as key_file:
+            private_key = cose_key.from_pem(key_file.read(), alg="RS256")
+        with open(key_path("public_key_rsa.pem")) as key_file:
+            public_key = cose_key.from_pem(key_file.read(), alg="RS256")
+
+        token = cwt.encode(
+            {"iss": "coaps://as.example", "sub": "dajiaji", "cti": "123"}, private_key
+        )
+
+        decoded = cwt.decode(token, public_key)
+        assert 1 in decoded and decoded[1] == "coaps://as.example"
+
+    def test_sample_readme_signed_cwt_ps256(self):
+        with open(key_path("private_key_rsa.pem")) as key_file:
+            private_key = cose_key.from_pem(key_file.read(), alg="PS256")
+        with open(key_path("public_key_rsa.pem")) as key_file:
+            public_key = cose_key.from_pem(key_file.read(), alg="PS256")
+
+        token = cwt.encode(
+            {"iss": "coaps://as.example", "sub": "dajiaji", "cti": "123"}, private_key
+        )
+
+        decoded = cwt.decode(token, public_key)
+        assert 1 in decoded and decoded[1] == "coaps://as.example"
 
     def test_sample_readme_encrypted_cwt_old(self):
         nonce = token_bytes(13)
@@ -305,21 +332,21 @@ class TestSample:
         enc_key = cose_key.from_symmetric_key(mysecret, alg="AES-CCM-16-64-256")
         encoded = cwt.encode_and_encrypt(
             claims.from_json(
-                {"iss": "https://as.example", "sub": "dajiaji", "cti": "123"}
+                {"iss": "coaps://as.example", "sub": "dajiaji", "cti": "123"}
             ),
             enc_key,
             nonce=nonce,
         )
         decoded = cwt.decode(encoded, enc_key)
-        assert 1 in decoded and decoded[1] == "https://as.example"
+        assert 1 in decoded and decoded[1] == "coaps://as.example"
 
     def test_sample_readme_encrypted_cwt(self):
         enc_key = cose_key.from_symmetric_key(alg="ChaCha20/Poly1305")
         token = cwt.encode(
-            {"iss": "https://as.example", "sub": "dajiaji", "cti": "123"}, enc_key
+            {"iss": "coaps://as.example", "sub": "dajiaji", "cti": "123"}, enc_key
         )
         decoded = cwt.decode(token, enc_key)
-        assert 1 in decoded and decoded[1] == "https://as.example"
+        assert 1 in decoded and decoded[1] == "coaps://as.example"
 
     def test_sample_readme_nested_cwt_old(self):
         with open(key_path("private_key_es256.pem")) as key_file:
@@ -329,7 +356,7 @@ class TestSample:
 
         encoded = cwt.encode_and_sign(
             claims.from_json(
-                {"iss": "https://as.example", "sub": "dajiaji", "cti": "123"}
+                {"iss": "coaps://as.example", "sub": "dajiaji", "cti": "123"}
             ),
             private_key,
         )
@@ -340,7 +367,7 @@ class TestSample:
         nested = cwt.encode_and_encrypt(encoded, enc_key, nonce=nonce)
 
         decoded = cwt.decode(nested, [enc_key, public_key])
-        assert 1 in decoded and decoded[1] == "https://as.example"
+        assert 1 in decoded and decoded[1] == "coaps://as.example"
 
     def test_sample_readme_nested_cwt(self):
         with open(key_path("private_key_es256.pem")) as key_file:
@@ -349,14 +376,205 @@ class TestSample:
             public_key = cose_key.from_pem(key_file.read())
 
         token = cwt.encode(
-            {"iss": "https://as.example", "sub": "dajiaji", "cti": "124"}, private_key
+            {"iss": "coaps://as.example", "sub": "dajiaji", "cti": "123"}, private_key
         )
 
         enc_key = cose_key.from_symmetric_key(alg="ChaCha20/Poly1305")
         nested = cwt.encode(token, enc_key)
 
         decoded = cwt.decode(nested, [enc_key, public_key])
-        assert 1 in decoded and decoded[1] == "https://as.example"
+        assert 1 in decoded and decoded[1] == "coaps://as.example"
+
+    def test_sample_readme_cwt_with_pop_jwk(self):
+
+        # issuer:
+        with open(key_path("private_key_ed25519.pem")) as key_file:
+            private_key = cose_key.from_pem(key_file.read())
+        token = cwt.encode(
+            {
+                "iss": "coaps://as.example",
+                "sub": "dajiaji",
+                "cti": "123",
+                "cnf": {
+                    "jwk": {
+                        "kty": "OKP",
+                        "use": "sig",
+                        "crv": "Ed25519",
+                        "kid": "01",
+                        "x": "2E6dX83gqD_D0eAmqnaHe1TC1xuld6iAKXfw2OVATr0",
+                        "alg": "EdDSA",
+                    },
+                },
+            },
+            private_key,
+        )
+
+        # presenter:
+        msg = b"could-you-sign-this-message?"  # Provided by recipient.
+        pop_key_private = cose_key.from_jwk(
+            {
+                "kty": "OKP",
+                "d": "L8JS08VsFZoZxGa9JvzYmCWOwg7zaKcei3KZmYsj7dc",
+                "use": "sig",
+                "crv": "Ed25519",
+                "kid": "01",
+                "x": "2E6dX83gqD_D0eAmqnaHe1TC1xuld6iAKXfw2OVATr0",
+                "alg": "EdDSA",
+            }
+        )
+        sig = pop_key_private.sign(msg)
+
+        # recipient:
+        with open(key_path("public_key_ed25519.pem")) as key_file:
+            public_key = cose_key.from_pem(key_file.read())
+        decoded = cwt.decode(token, public_key)
+        assert 8 in decoded and isinstance(decoded[8], dict)
+        assert 1 in decoded[8] and isinstance(decoded[8][1], dict)
+        c = claims.from_dict(decoded)
+        extracted = cose_key.from_dict(c.cnf)
+        try:
+            extracted.verify(msg, sig)
+        except Exception:
+            pytest.fail("verify should not fail.")
+
+    def test_sample_readme_cwt_with_pop_encrypted_cose_key_readable(self):
+        with open(key_path("private_key_ed25519.pem")) as key_file:
+            private_key = cose_key.from_pem(key_file.read())
+        enc_key = cose_key.from_symmetric_key(
+            "a-client-secret-of-cwt-recipient",  # Just 32 bytes!
+            alg="ChaCha20/Poly1305",
+        )
+        pop_key = cose_key.from_symmetric_key(
+            "a-client-secret-of-cwt-presenter",
+            alg="HMAC 256/256",
+        )
+        token = cwt.encode(
+            {
+                "iss": "coaps://as.example",
+                "sub": "dajiaji",
+                "cti": "123",
+                "cnf": {
+                    # 'eck'(Encrypted Cose Key) is a keyword defined by this library.
+                    "eck": cose_key.to_encrypted_cose_key(pop_key, enc_key),
+                },
+            },
+            private_key,
+        )
+
+        with open(key_path("public_key_ed25519.pem")) as key_file:
+            public_key = cose_key.from_pem(key_file.read())
+        decoded = cwt.decode(token, public_key)
+        assert 8 in decoded and isinstance(decoded[8], dict)
+        assert 2 in decoded[8] and isinstance(decoded[8][2], list)
+        c = claims.from_dict(decoded)
+        extracted = cose_key.from_encrypted_cose_key(c.cnf, enc_key)
+        assert extracted.kty == 4  # Symmetric
+        assert extracted.alg == 5  # HMAC 256/256
+        assert extracted.key == b"a-client-secret-of-cwt-presenter"
+
+    def test_sample_readme_cwt_with_pop_kid_readable(self):
+        with open(key_path("private_key_ed25519.pem")) as key_file:
+            private_key = cose_key.from_pem(key_file.read())
+
+        token = cwt.encode(
+            {
+                "iss": "coaps://as.example",
+                "sub": "dajiaji",
+                "cti": "123",
+                "cnf": {
+                    "kid": "pop-key-id-of-cwt-presenter",
+                },
+            },
+            private_key,
+        )
+
+        with open(key_path("public_key_ed25519.pem")) as key_file:
+            public_key = cose_key.from_pem(key_file.read())
+        decoded = cwt.decode(token, public_key)
+        assert 8 in decoded and isinstance(decoded[8], dict)
+        assert 3 in decoded[8] and decoded[8][3] == b"pop-key-id-of-cwt-presenter"
+        c = claims.from_dict(decoded)
+        assert c.cnf == "pop-key-id-of-cwt-presenter"
+
+    def test_sample_readme_cwt_with_pop_cose_key(self):
+        with open(key_path("private_key_ed25519.pem")) as key_file:
+            private_key = cose_key.from_pem(key_file.read())
+        with open(key_path("public_key_es256.pem")) as key_file:
+            pop_key = cose_key.from_pem(key_file.read())
+        token = cwt.encode(
+            {
+                1: "coaps://as.example",  # iss
+                2: "dajiaji",  # sub
+                7: b"123",  # cti
+                8: {  # cnf
+                    1: pop_key.to_dict(),
+                },
+            },
+            private_key,
+        )
+
+        with open(key_path("public_key_ed25519.pem")) as key_file:
+            public_key = cose_key.from_pem(key_file.read())
+        decoded = cwt.decode(token, public_key)
+        assert 8 in decoded and isinstance(decoded[8], dict)
+        assert 1 in decoded[8] and isinstance(decoded[8][1], dict)
+        extracted = cose_key.from_dict(decoded[8][1])
+        assert extracted.kty == 2  # EC2
+        assert extracted.crv == 1  # P-256
+
+    def test_sample_readme_cwt_with_pop_encrypted_cose_key(self):
+        with open(key_path("private_key_ed25519.pem")) as key_file:
+            private_key = cose_key.from_pem(key_file.read())
+        enc_key = cose_key.from_symmetric_key(
+            "a-client-secret-of-cwt-recipient",  # Just 32 bytes!
+            alg="ChaCha20/Poly1305",
+        )
+        pop_key = cose_key.from_symmetric_key(
+            "a-client-secret-of-cwt-presenter",
+            alg="HMAC 256/256",
+        )
+        token = cwt.encode(
+            {
+                1: "coaps://as.example",  # iss
+                2: "dajiaji",  # sub
+                7: b"123",  # cti
+                8: {  # cnf
+                    2: cose_key.to_encrypted_cose_key(pop_key, enc_key),
+                },
+            },
+            private_key,
+        )
+
+        with open(key_path("public_key_ed25519.pem")) as key_file:
+            public_key = cose_key.from_pem(key_file.read())
+        decoded = cwt.decode(token, public_key)
+        assert 8 in decoded and isinstance(decoded[8], dict)
+        assert 2 in decoded[8] and isinstance(decoded[8][2], list)
+        extracted = cose_key.from_encrypted_cose_key(decoded[8][2], enc_key)
+        assert extracted.kty == 4  # Symmetric
+        assert extracted.alg == 5  # HMAC 256/256
+        assert extracted.key == b"a-client-secret-of-cwt-presenter"
+
+    def test_sample_readme_cwt_with_pop_kid(self):
+        with open(key_path("private_key_ed25519.pem")) as key_file:
+            private_key = cose_key.from_pem(key_file.read())
+        token = cwt.encode(
+            {
+                1: "coaps://as.example",  # iss
+                2: "dajiaji",  # sub
+                7: b"123",  # cti
+                8: {  # cnf
+                    3: b"pop-key-id-of-cwt-presenter",
+                },
+            },
+            private_key,
+        )
+
+        with open(key_path("public_key_ed25519.pem")) as key_file:
+            public_key = cose_key.from_pem(key_file.read())
+        decoded = cwt.decode(token, public_key)
+        assert 8 in decoded and isinstance(decoded[8], dict)
+        assert 3 in decoded[8] and decoded[8][3] == b"pop-key-id-of-cwt-presenter"
 
     def test_sample_rfc8392_a3(self):
         key = cose_key.from_bytes(bytes.fromhex(SAMPLE_COSE_KEY_RFC8392_A2_3))
